<?xml version="1.0" encoding="utf-8"?>
<!--
  ** Author: Marco Ladermann
  ** Date: Thu Sep 12 18:57:21 CEST 2002 @831 /Internet Time/
  ** Purpose:
  ** Changed:
  **
  ** This software is free software. It is released under the terms of the
  ** GNU Lesser General Public Licence (LGPL)
  ** see http://www.gnu.org/copyleft/lesser.html
  **
  ** These stylesheets are distributed in the hope that they will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
  ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  -->
<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:func="http://exslt.org/functions"
    xmlns:str="http://exslt.org/strings"
    xmlns:kde="http://kde.org/functions"
    extension-element-prefixes="func"
>
    <!-- Function: kde:if
      ** @param bool
      ** @param trueVal
      ** @param falseVal
      ** @return ( bool ) ? trueVal : falseVal
      -->
    <func:function name="kde:if">
        <xsl:param name="bool"/>
        <xsl:param name="trueVal"/>
        <xsl:param name="falseVal"/>
        <func:result>
            <xsl:choose>
                <xsl:when test="$bool"><xsl:value-of select="$trueVal"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="$falseVal"/></xsl:otherwise>
            </xsl:choose>
        </func:result>
    </func:function>

    <!-- Function: kde:index
      ** @param string
      ** @param substring
      ** @return the index of "substring" in "string"
      -->
    <func:function name="kde:index">
        <xsl:param name="string"/>
        <xsl:param name="substring"/>
        <func:result select="string-length(substring-before($string, $substring))"/> 
    </func:function>
    
    <!--
      ** Function: replace
      ** @param string1 to search
      ** @param string2 to look up
      ** @param string3 to substitute
      ** @return $string1 with all occureces of $string2 replaced by $string3 
      -->
    <func:function name="kde:replace">
        <xsl:param name="string1"/>
        <xsl:param name="string2"/>
        <xsl:param name="string3"/>
        <func:result>
            <xsl:choose>
                <xsl:when test="function-available('str:replace')">
                    <xsl:value-of select="str:replace($string1, $string2, $string3)"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="kde:replace-string($string1, $string2, $string3)"/>
                </xsl:otherwise>
            </xsl:choose>
        </func:result>
    </func:function>
    <func:function name="kde:replace-string">
        <xsl:param name="string1"/>
        <xsl:param name="string2"/>
        <xsl:param name="string3"/>
        <func:result>
            <xsl:variable name="first" select="substring-before($string1, $string2)"/>
            <xsl:variable name="tail" select="substring-after($string1, $string2)"/>
            <xsl:choose>
                <xsl:when test="$first = ''">
                    <xsl:value-of select="$string1"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat($first,
                                                 $string3,
                                                 kde:replace-string($tail, $string2, $string3))"/>
                </xsl:otherwise>
            </xsl:choose>
        </func:result> 
    </func:function>
    
    <!--
      ** Function: upper-case
      ** @param string
      ** @return string in upper case letter
      -->
    <func:function name="kde:upper-case">
        <xsl:param name="string"/>
        <func:result select="translate($string, 
                                       'abcdefghijklmnopqrstuvwxyz',
                                       'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/> 
    </func:function>
    

    <!--
      ** Function: lower-case
      ** @param string
      ** @return string in upper case letter
      -->
    <func:function name="kde:lower-case">
        <xsl:param name="string"/>
        <func:result select="translate($string, 
                                       'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 
                                       'abcdefghijklmnopqrstuvwxyz')"/>
    </func:function>
    
    <!--
      ** Function: upper-first
      ** @param string
      ** @return string with first character as upper case letter
      -->
    <func:function name="kde:upper-first">
        <xsl:param name="string"/>
        <xsl:variable name="rest" select="substring($string, 2)"/>
        <xsl:variable name="head" select="kde:upper-case(substring($string, 1, 1))"/>
        <func:result>
            <xsl:value-of select="concat($head, $rest)"/>
        </func:result>    
    </func:function>

    <!--
      ** Function: repeat
      ** @param string
      ** @param times
      ** @return 'times' copies of string
      -->
    <func:function name="kde:repeat">
        <xsl:param name="string"/>
        <xsl:param name="times"/>
        <xsl:variable name="copy" select="$string"/>
        <func:result>
            <xsl:choose>
                <xsl:when test="$times &lt;= 0"><xsl:value-of select="''"/></xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat($string, kde:repeat($copy, $times - 1))"/>
                </xsl:otherwise>
            </xsl:choose>
        </func:result>
    </func:function>
    
    <!--
      ** Function: getNodeName
      ** @param node
      ** @return the (variable) name of a node 
      -->
    <func:function name="kde:getNodeName">
        <xsl:param name="node" select="."/>
        <func:result>
            <xsl:variable name="name" select="$node/property[@name = 'name']/cstring"/>
            <xsl:choose>
                <xsl:when test="$name = 'unnamed'">
                    <xsl:value-of select="concat(kde:getNodeName($node/..), kde:upper-first(name($node)))"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$name"/>
                </xsl:otherwise>
            </xsl:choose>
        </func:result>
    </func:function>

    <!--
      ** Templates to "print" xml in text output mode
    -->
    <xsl:template match="*" mode="print">
        <xsl:value-of select="concat('&lt;', name())"/>
        <xsl:apply-templates select="@*" mode="print"/>
        <xsl:value-of select="'&gt;'"/>
        <xsl:apply-templates mode="print"/> 
        <xsl:value-of select="concat('&lt;/', name(), '&gt;')"/>
    </xsl:template>
    <xsl:template match="@*" mode="print">
        <xsl:value-of select="concat(' ', name(), '=&quot;', ., '&quot;')"/>
    </xsl:template>
    
    
    <!--
      ** Print the path from the root to a given node
      -->
    <func:function name="kde:printPath">
        <xsl:param name="node"/>
        <xsl:param name="path" select="''"/>
        <func:result>
            <xsl:choose>
                <xsl:when test="$node = /UI">
                    <xsl:value-of select="concat('/UI', $path)"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="name" select="name($node)"/>
                    <xsl:variable name="pos" select="count($node/preceding-sibling::*[name() = $name]) + 1"/>
                    <xsl:value-of select="kde:printPath($node/.., concat('/',
                                                                         $name, 
                                                                         '[', $pos, ']',
                                                                         $path))"/>
                </xsl:otherwise>
            </xsl:choose>
        </func:result>
    </func:function>
    


</xsl:stylesheet>

